/* demux -- simple demux tool for the Linux DVB S2 API
 *
 * UDL (updatelee@gmail.com)
 * Derived from work by:
 * 	Igor M. Liplianin (liplianin@me.by)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "demux.h"

#if DVB_API_VERSION < 5
#error tune-s2 requires Linux DVB driver API version 5.0 or newer!
#endif

#define BUFFY (188*20)
#define MAX_LENGTH (1024*1024*5) /* record 5MB */

char *usage =
	"\n"
	"   usage: supply a vpid, apid, pcr, and pmt pid to create a vlc playable ts file \n"
	"          demux [options] -pids 1111 2222 333 444 \n"
	"	-adapter N     : use given adapter (default 0)\n"
	"	-filename      : use filename insted of /tmp/test.ts\n"
	"   -pids          : use to supply the app with the list of pids you want saved \n" 
	"	-help          : help\n";

int main(int argc, char *argv[])
{
	char demux_devname[80];
	char dvr_devname[80];
	char *dvr_filename = "/tmp/test.ts";
	int adapter = 0;
	int demux[20];
	int pids[20];
	int num_pids = 1;
	pids[0] = 0;
		
	int a, p;
	for( a = 1; a < argc; a++ )
	{
		if ( !strcmp(argv[a], "-adapter") )
			adapter = strtoul(argv[a+1], NULL, 0);
		if ( !strcmp(argv[a], "-filename") )
			dvr_filename = argv[a+1];
		if ( !strcmp(argv[a], "-help") )
		{
			printf("%s", usage);
			return -1;
		}
		if ( !strcmp(argv[a], "-pids") )
			for( p = a+1 ; p < argc; p++ )
			{
				pids[num_pids] = strtoul(argv[p], NULL, 0);
				num_pids++;
			}
	}
	if (num_pids == 1)
		pids[0] = 8192;

	snprintf(demux_devname, 80, "/dev/dvb/adapter%d/demux0", adapter);
	printf("opening: %s\n", demux_devname);
	snprintf(dvr_devname, 80, "/dev/dvb/adapter%d/dvr0", adapter);
	printf("opening: %s\n", dvr_devname);

	int i;
	int len;
	int written;
	uint8_t buf[BUFFY];
	uint64_t length;
	struct pollfd pfd[1];

	int dvr_fd;
	if ((dvr_fd = open(dvr_devname, O_RDONLY|O_NONBLOCK)) < 0)
	{
		printf("failed to open '%s': %d %m\n", dvr_devname, errno);
		return -1;
	}

	struct dmx_pes_filter_params pesFilterParams;
	for( p = 0 ; p < num_pids; p++ )
	{
		printf("pid #%d: %d \n", p, pids[p]);
		if ((demux[p] = open(demux_devname, O_RDWR|O_NONBLOCK)) < 0)
		{
			printf("failed to open demux %d '%s': %d %m\n", p, demux_devname, errno);
			return -1;
		}
		pesFilterParams.pid = pids[p];
		pesFilterParams.input = DMX_IN_FRONTEND; 
		pesFilterParams.output = DMX_OUT_TS_TAP; 
		pesFilterParams.pes_type = DMX_PES_OTHER; 
		pesFilterParams.flags = DMX_IMMEDIATE_START;
		if (ioctl(demux[p], DMX_SET_PES_FILTER, &pesFilterParams) == -1){
			perror("DEMUX1: DMX_SET_PES_FILTER");
			return -1; 
		}
	}

	printf("Recording %2.0f MB of test file in TS format\n", MAX_LENGTH/(1024.0*1024.0));
	length = 0;

	/* open output file */
	int dvr_out;
	if ((dvr_out = open(dvr_filename, O_WRONLY|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH)) < 0){
		printf("failed to open '%s': %d %m\n", dvr_filename, errno);
		return -1;
	}

	pfd[0].fd = dvr_fd;
	pfd[0].events = POLLIN;

    int c;
    set_conio_terminal_mode();
    do {
		if (poll(pfd,1,1))
		{
			if (pfd[0].revents & POLLIN)
			{
				len = read(dvr_fd, buf, BUFFY);
				if (len > 0)
				{
					written = 0;
					while (written < len) 
						written += write(dvr_out, buf, len);
					length += len;
					printf("written %2.0f MB\r", length/1024./1024.);
				} else {
					perror("recording");
					return -1;
				}
			}
		}
		if ( kbhit() )
	   		c = kbgetchar(); /* consume the character */
    } while( (char)c != 'q' );

	printf("Closing demux ... \n");
	close(dvr_out);
	close(dvr_fd);
	for( p = 0 ; p < num_pids; p++ )
		close(demux[p]);
	return 0;
}
